# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/26 3:56 下午
# Copyright (C) 2021 The lesscode Team

from lesscode.db.base_sql_helper import BaseSqlHelper, echo_sql
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.page import Page
from lesscode.db.relational_db_helper import RelationalDbHelper


class PostgresqlHelper(RelationalDbHelper):
    """
    Postgresql数据库操作实现
    """

    @echo_sql
    async def execute_sql(self, sql: str, param=None):
        """
        执行sql 仅返回影响数量
        :param sql:
        :param param:
        :return:
        """
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cursor:
                await cursor.execute(sql, param)
                rowcount = cursor.rowcount
                result = {"rowcount": rowcount}
                description = cursor.description
                if description:
                    rs = await cursor.fetchone()
                    result["result"] = BaseSqlHelper.dict_row(description, rs)
        return result

    @echo_sql
    async def executemany_sql(self, sql: str, param=None):
        """
        执行批量sql 仅返回影响数量
        :param sql:
        :param param:
        :return:
        """
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cursor:
                # 执行sql 传入参数
                await cursor.executemany(sql, [tuple(item.values()) for item in param])
        return {"rowcount": cursor.rowcount}

    @echo_sql
    async def execute_fetchone(self, sql: str, param=None):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                if param:
                    await cur.execute(sql, param)
                else:
                    await cur.execute(sql)
                description = cur.description
                rs = await cur.fetchone()
        return BaseSqlHelper.dict_row(description, rs)

    @echo_sql
    async def execute_fetchall(self, sql: str, param=None):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        async with self.pool.acquire() as conn:
            async with conn.cursor() as cur:
                if param:
                    await cur.execute(sql, param)
                else:
                    await cur.execute(sql)
                description = cur.description
                rs = await cur.fetchall()
        return BaseSqlHelper.dict_row(description, rs)

    @echo_sql
    def sync_execute_sql(self, sql: str, param=None):
        """
        执行sql 仅返回影响数量
        :param sql:
        :param param:
        :return:
        """
        with self.pool.connection() as conn:
            with conn.cursor() as cursor:
                cursor.execute(sql, param)
                rowcount = cursor.rowcount
                result = {"rowcount": rowcount}
                description = cursor.description
                if description:
                    rs = cursor.fetchone()
                    result["result"] = BaseSqlHelper.dict_row(description, rs)
            conn.commit()
        return result

    @echo_sql
    def sync_executemany_sql(self, sql: str, param=None):
        """
        执行批量sql 仅返回影响数量
        :param sql:
        :param param:
        :return:
        """
        with self.pool.connection() as conn:
            with conn.cursor() as cursor:
                cursor.executemany(sql, [tuple(item.values()) for item in param])
            conn.commit()
        return {"rowcount": cursor.rowcount}

    @echo_sql
    def sync_execute_fetchone(self, sql: str, param=None):
        """
        查询单条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        with self.pool.connection() as conn:
            with conn.cursor() as cursor:
                if param:
                    cursor.execute(sql, param)
                else:
                    cursor.execute(sql)
                description = cursor.description
                rs = cursor.fetchone()
        return BaseSqlHelper.dict_row(description, rs)

    @echo_sql
    def sync_execute_fetchall(self, sql: str, param=None):
        """
        查询多条数据
        :param sql: 待执行的Sql语句
        :param param: 参数
        :return:
        """
        with self.pool.connection() as conn:
            with conn.cursor() as cursor:
                if param:
                    cursor.execute(sql, param)
                else:
                    cursor.execute(sql)
                description = cursor.description
                rs = cursor.fetchall()
        return BaseSqlHelper.dict_row(description, rs)

    def prepare_page_sql(self, condition_wrapper: ConditionWrapper, page_num: int, page_size: int):
        """
        获取分页SQL
        :param condition_wrapper: 查询参数
        :param page_num: 当前页
        :param page_size: 每页记录数
        :return:page_sql 分页sql, values 对应参数, count_sql 统计总数sql
        """
        query_sql, values = self.prepare_query_sql(condition_wrapper)
        cols = []
        for col in condition_wrapper.column:
            col = col.upper().replace("AS", "").strip().split(" ")
            cols.append(col[-1].strip())
        # 组装分页查询语句，需要依据页码计算 起始索引
        page_sql = f"SELECT {','.join(cols)} FROM ({query_sql}) cs OFFSET {Page.skip(page_num, page_size)} LIMIT {page_size}"
        # 组装查询数量语句
        count_sql = f"SELECT count(1) total FROM ({query_sql}) cs"
        return page_sql, values, count_sql

    def prepare_insert_sql(self, table_name: str, item: dict):
        """
        组装插入sql
        :param table_name:
        :param item:
        :return:
        """
        # 组装可执行SQL
        insert_sql = super().prepare_insert_sql(table_name, item)
        return f"{insert_sql} RETURNING *"

    def prepare_update_sql(self, condition_wrapper: ConditionWrapper, param: dict):
        """
        组装更新sql
        :param condition_wrapper:
        :param param:
        :return:
        """
        # 组装可执行SQL
        update_sql, values = super().prepare_update_sql(condition_wrapper, param)
        return f"{update_sql} RETURNING *", values



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/10 3:57 下午
# Copyright (C) 2021 The lesscode Team

from datetime import datetime

from lesscode.web.status_code import StatusCode


class ResponseResult(dict):
    """
    ResponseResult 类用于统一包装数据返回格式
    """
    def __init__(self, status_code=StatusCode.SUCCESS, data=""):
        super(ResponseResult, self).__init__()
        # 业务请求状态编码
        self["status"] = status_code[0]
        # 返回状态码对应的说明信息
        self["message"] = status_code[1]
        # 返回数据对象 主对象 指定类型
        self["data"] = data
        # 时间戳
        self["timestamp"] = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')



import json
import aiohttp
import tornado.options
import requests
import py_eureka_client.eureka_client as eureka_client


async def post(path, data=None,
               base_url=tornado.options.options.data_server,
               result_type="json", pack=False, **kwargs):
    flag = kwargs.pop("flag", True)
    if flag:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Content-Type": "application/json"
            }})
    async with aiohttp.ClientSession() as session:
        flag = kwargs.pop("flag", True)
        if flag:
            async with session.post(base_url + path, json=data, **kwargs) as resp:
                if result_type == "json":
                    result = await resp.json()
                    if result.get("status") == "00000" and not pack:
                        result = result.get("data")
                elif result_type == "text":
                    result = await resp.text()
                else:
                    result = await resp.content.read()
                return result
        else:
            async with session.post(base_url + path, data=data, **kwargs) as resp:
                if result_type == "json":
                    result = await resp.json()
                    if result.get("status") == "00000" and not pack:
                        result = result.get("data")
                elif result_type == "text":
                    result = await resp.text()
                else:
                    result = await resp.content.read()
                return result


async def get(path, params=None, base_url=tornado.options.options.data_server, result_type="json", pack=False,
              **kwargs):
    async with aiohttp.ClientSession() as session:
        async with session.get(base_url + path, params=params, **kwargs) as resp:
            if result_type == "json":
                result = await resp.json()
                if result.get("status") == "00000" and not pack:
                    result = result.get("data")
            elif result_type == "text":
                result = await resp.text()
            else:
                result = await resp.content.read()
            return result


def sync_get(path, params=None, base_url=tornado.options.options.data_server, result_type="json", pack=False, **kwargs):
    res = requests.get(base_url + path, params=params, **kwargs)
    if result_type == "json":
        res = res.json()
        if res.get("status") == "00000" and not pack:
            res = res.get("data")
    elif result_type == "text":
        res = res.text
    else:
        res = res.content
    return res


def sync_post(path, data=None,
              base_url=tornado.options.options.data_server,
              result_type="json", pack=False, **kwargs):
    flag = kwargs.pop("flag", True)
    if flag:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Content-Type": "application/json"
            }})
        res = requests.post(base_url + path, json=data, **kwargs)
    else:
        res = requests.post(base_url + path, data=data, **kwargs)
    if result_type == "json":
        res = res.json()
        if res.get("status") == "00000" and not pack:
            res = res.get("data")
    elif result_type == "text":
        res = res.text
    else:
        res = res.content
    return res


def eureka_request(path="", return_type="json", method="GET", data=None, app_name="DATA_SERVICE", pack=False, **kwargs):
    flag = kwargs.pop("flag", True)
    if flag:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Content-Type": "application/json"
            }})
    res = eureka_client.do_service(app_name=app_name, service=path, return_type=return_type, method=method, data=data,
                                   **kwargs)
    if return_type == "json" and not pack:
        if res.get("status") == "00000" and not pack:
            res = res.get("data")
    return res


def eureka_get(path="", data=None, return_type="json", app_name="DATA_SERVICE", pack=False, **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")

    if tornado.options.options.running_env != "local":

        res = eureka_request(path=path, return_type=return_type, method="GET", data=data, app_name=app_name, pack=pack,
                             **kwargs)
    else:
        res = sync_get(path=path, params=data, base_url=base_url,
                       result_type=return_type,
                       pack=pack, **kwargs)
    return res


def eureka_post(path="", data=None, return_type="json", app_name="DATA_SERVICE", pack=False, **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.running_env != "local":

        res = eureka_request(path=path, return_type=return_type, method="POST", data=json.dumps(data),
                             app_name=app_name,
                             pack=pack, **kwargs)
    else:
        res = sync_post(path=path, data=data, base_url=base_url,
                        result_type=return_type,
                        pack=pack, **kwargs)
    return res



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/26 2:08 下午
# Copyright (C) 2021 The lesscode Team
from math import ceil


class Page:

    def __init__(self, total, current: int, page_size: int, records=None):
        self.records = records
        # 总记录数
        self.total = int(total)
        # 每页显示条数，默认 10
        self.page_size = int(page_size)
        # 当前页数
        self.current = int(current)
        # 总页数
        self.pages = ceil(self.total / int(self.page_size))
        # 是否存在前一页
        self.hasPrevious: bool = int(self.current) > 1
        # 是否存在下一页
        self.hasNext: bool = int(self.current) < int(self.pages)

    @staticmethod
    def skip(page_num: int, page_size: int):
        """
        依据页码与条数 计算跳过索引
        :param page_num:
        :param page_size:
        :return:
        """
        return (int(page_num) - 1) * int(page_size)



import functools
import json

import tornado.options

from lesscode.web.business_exception import BusinessException
from lesscode.web.status_code import StatusCode


def user_verification(username="admin", **kw):
    def wrapper(func):
        @functools.wraps(func)
        async def run(self, *args, **kwargs):
            if tornado.options.options.running_env != "local":
                user_str = self.request.headers.get("User")
                if user_str:
                    user = json.loads(user_str)
                    if isinstance(user, dict):
                        user_username = user.get("username")
                        if username != user_username:
                            raise BusinessException(StatusCode.ACCESS_DENIED)
                    else:
                        raise BusinessException(StatusCode.ACCESS_DENIED)
                else:
                    raise BusinessException(StatusCode.ACCESS_DENIED)

            return await func(self, *args, **kwargs)

        return run

    return wrapper



import math
import os

import tornado.options
from filechunkio import FileChunkIO
from ks3.connection import Connection
from ks3.key import Key
from ks3.prefix import Prefix


class Ks3Oss:
    def __init__(self, **kwargs):
        if not kwargs.get("access_key_id"):
            kwargs.update({"access_key_id": tornado.options.options.ks3_connect_config.get("access_key_id")})
        if not kwargs.get("access_key_secret"):
            kwargs.update({"access_key_secret": tornado.options.options.ks3_connect_config.get("access_key_secret")})
        if not kwargs.get("host"):
            kwargs.update({"host": tornado.options.options.ks3_connect_config.get("host")})
        bucket_name = kwargs.pop("bucket_name", None)
        self.client = Connection(**kwargs)
        if bucket_name is not None:
            self.instance = self.client.get_bucket(bucket_name)
        else:
            self.instance = None

    def create_bucket(self, bucket_name, **kwargs):
        bucket = self.client.create_bucket(bucket_name=bucket_name, **kwargs)
        return bucket

    def list_buckets(self):
        buckets = self.client.get_all_buckets()
        return [b.name for b in buckets]

    def delete_bucket(self, bucket_name):
        return self.client.delete_bucket(bucket_name)

    def save(self, key, bucket_name="", content_type="string", protocol="https", domain="ksyun.com",
             region="ks3-cn-beijing", **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        k = self.instance.new_key(key)
        ret = None
        if content_type == "string":
            ret = k.set_contents_from_string(**kwargs)
        elif content_type == "file":
            ret = k.set_contents_from_file(**kwargs)
        elif content_type == "filename":
            ret = k.set_contents_from_filename(**kwargs)
        elif content_type == "network":
            ret = k.fetch_object(**kwargs)
        if ret:
            if ret.status == 200:
                return f'{protocol}://{bucket_name}.{region}.{domain}/{key}'
            else:
                return False
        else:
            return False

    def get_url(self, key, bucket_name="", region="ks3-cn-beijing", domain="ksyun.com", protocol="https"):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        if self.instance.get_key(key):
            return f'{protocol}://{bucket_name}.{region}.{domain}/{key}'
        else:
            return None

    def get_file(self, key, bucket_name="", file_path=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        k = self.instance.get_key(key)
        if file_path:
            k.get_contents_to_filename(file_path)
            return True
        else:
            contents = k.get_contents_as_string().decode()
            return contents

    def get_key(self, key, bucket_name=""):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        k = self.instance.get_key(key)
        return k

    def delete_file(self, key, bucket_name=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        res = self.instance.delete_key(key)
        return res

    def list_file(self, bucket_name, delimiter="/"):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        keys = self.instance.list(delimiter=delimiter)
        files = list()
        dirs = list()
        for k in keys:
            if isinstance(k, Key):
                files.append(k.name)
            elif isinstance(k, Prefix):
                dirs.append(k.name)

        for p in dirs:
            keys = self.instance.list(prefix=p)
            for k in keys:
                if isinstance(k, Key):
                    files.append(k.name)
                elif isinstance(k, Prefix):
                    dirs.append(k.name)
        return dirs, files

    def get_bucket_acl(self, bucket_name=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        acl = self.instance.get_acl()
        return acl

    def set_bucket_acl(self, bucket_name=None, *args, **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        return self.instance.set_acl(*args, **kwargs)

    def get_bucket_policy(self, bucket_name=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        policy = self.instance.get_bucket_policy()
        return policy

    def set_bucket_policy(self, policy, bucket_name=None, headers=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        return self.instance.set_bucket_policy(policy, headers)

    def delete_bucket_policy(self, bucket_name=None, headers=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        return self.instance.delete_bucket_policy(headers)

    def multipart_upload(self, file_path, chunk_size, bucket_name=None, **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        file_size = os.stat(file_path).st_size
        mp = self.instance.initiate_multipart_upload(os.path.basename(file_path), **kwargs)
        chunk_count = int(math.ceil(file_size * 1.0 / chunk_size * 1.0))
        for i in range(chunk_count):
            offset = chunk_size * i
            bytes = min(chunk_size, file_size - offset)
            with FileChunkIO(file_size, 'r', offset=offset, bytes=bytes) as fp:
                mp.upload_part_from_file(fp, part_num=i + 1)
        ret = mp.complete_upload()
        return ret

    def list_multipart_upload_parts(self, key, bucket_name=None, **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        mp = self.instance.initiate_multipart_upload(key, **kwargs)
        return mp.get_all_parts()

    def list_multipart_uploads(self, key, bucket_name=None, **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        return self.instance.list_multipart_uploads(key, **kwargs)

    def generate_url(self, key, second, bucket_name=None):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        k = self.instance.get_key(key)
        if k:
            url = k.generate_url(second)
            return url
        else:
            return None

    def get_presigned_url(self, key, second, bucket_name=None, expires_in_absolute=False):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        k = self.instance.new_key(key)
        if k:
            url = k.get_presigned_url(second, expires_in_absolute=expires_in_absolute)
            return url
        else:
            return None

    def modify_object(self, bucket_name=None, *args, **kwargs):
        if self.instance is None:
            self.instance = self.client.get_bucket(bucket_name)
        return self.instance.copy_key(*args, **kwargs)



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/26 3:56 下午
# Copyright (C) 2021 The lesscode Team
import aioredis
import redis
from tornado.options import options


class RedisHelper:

    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.dialect = options.database[pool]
        else:
            self.pool = pool

    def get_connection(self, sync=False):
        if sync:
            return redis.Redis(connection_pool=self.pool, decode_responses=True)
        else:
            return aioredis.Redis(connection_pool=self.pool, decode_responses=True)

    async def set(self, name, value, ex=None, px=None, nx: bool = False, xx: bool = False, keepttl: bool = False):
        await self.get_connection().set(name, value, ex, px, nx, xx, keepttl)

    async def get(self, name):
        return await self.get_connection().get(name)

    async def keys(self, pattern):
        return await self.get_connection().keys(pattern=pattern)

    async def exists(self, *name):
        return await self.get_connection().exists(*name)

    async def delete(self, names):
        if isinstance(names, list) or isinstance(names, tuple):
            await self.get_connection().delete(*names)
        else:
            await self.get_connection().delete(names)

    async def rpush(self, name, values: list, time=None):
        await self.get_connection().rpush(name, *values)
        if time:
            await self.get_connection().expire(name, time)

    async def hset(self, name, key=None, value=None, mapping=None, time=None):
        con = self.get_connection()
        await con.hset(name, key=key, value=value, mapping=mapping)
        if time:
            await con.expire(name, time)

    async def hgetall(self, name):
        return await self.get_connection().hgetall(name)

    async def hget(self, name, key):
        return await self.get_connection().hget(name, key)

    async def hdel(self, name, key):
        return await self.get_connection().hdel(name, key)

    async def hexists(self, name, key):
        return await self.get_connection().hexists(name, key)

    async def hincrby(self, name, key, amount: int):
        return await self.get_connection().hincrby(name, key, amount)

    async def hincrbyfloat(self, name, key, amount: float):
        return await self.get_connection().hincrbyfloat(name, key, amount)

    async def hkeys(self, name):
        return await self.get_connection().hkeys(name)

    async def hlen(self, name):
        return await self.get_connection().hlen(name)

    async def hmset(self, name, mapping, time=None):
        con = self.get_connection()
        await con.hmset(name, mapping)
        if time:
            await con.expire(name, time)

    async def hmget(self, name, keys, *args):
        return await self.get_connection().hmget(name, keys, *args)

    async def hsetnx(self, name, key, value):
        return await self.get_connection().hsetnx(name, key, value)

    async def hvals(self, name):
        return await self.get_connection().hvals(name)

    async def sadd(self, name, values: list, time=None):
        con = self.get_connection()
        await con.sadd(name, *values)
        if time:
            await con.expire(name, time)

    async def scard(self, name):
        return await self.get_connection().scard(name)

    async def sismember(self, name, value):
        return await self.get_connection().sismember(name, value)

    async def smembers(self, name):
        return await self.get_connection().smembers(name)

    async def spop(self, name, count):
        return await self.get_connection().spop(name, count)

    async def srem(self, name, *values):
        return await self.get_connection().srem(name, *values)

    def sync_set(self, name, value, ex=None, px=None, nx: bool = False, xx: bool = False, keepttl: bool = False):
        self.get_connection(sync=True).set(name, value, ex, px, nx, xx, keepttl)

    def sync_get(self, name):
        return self.get_connection(sync=True).get(name)

    def sync_keys(self, pattern):
        return self.get_connection(sync=True).keys(pattern=pattern)

    def sync_exists(self, *name):
        return self.get_connection(sync=True).exists(*name)

    def sync_delete(self, names):
        if isinstance(names, list) or isinstance(names, tuple):
            self.get_connection(sync=True).delete(*names)
        else:
            self.get_connection(sync=True).delete(names)

    def sync_rpush(self, name, values: list, time=None):
        con = self.get_connection(sync=True)
        con.rpush(name, *values)
        if time:
            con.expire(name, time)

    def sync_hset(self, name, key=None, value=None, mapping=None, time=None):
        con = self.get_connection(sync=True)
        con.hset(name, key=key, value=value, mapping=mapping)
        if time:
            con.expire(name, time)

    def sync_hgetall(self, name):
        return self.get_connection(sync=True).hgetall(name)

    def sync_hget(self, name, key):
        return self.get_connection(sync=True).hget(name, key)

    def sync_hdel(self, name, key):
        return self.get_connection(sync=True).hdel(name, key)

    def sync_hexists(self, name, key):
        return self.get_connection(sync=True).hexists(name, key)

    def sync_hincrby(self, name, key, amount: int):
        return self.get_connection(sync=True).hincrby(name, key, amount)

    def sync_hincrbyfloat(self, name, key, amount: float):
        return self.get_connection(sync=True).hincrbyfloat(name, key, amount)

    def sync_hkeys(self, name):
        return self.get_connection(sync=True).hkeys(name)

    def sync_hlen(self, name):
        return self.get_connection(sync=True).hlen(name)

    def sync_hmset(self, name, mapping, time=None):
        con = self.get_connection(sync=True)
        con.hmset(name, mapping)
        if time:
            con.expire(name, time)

    def sync_hmget(self, name, keys, *args):
        return self.get_connection(sync=True).hmget(name, keys, *args)

    def sync_hsetnx(self, name, key, value):
        return self.get_connection(sync=True).hsetnx(name, key, value)

    def sync_hvals(self, name):
        return self.get_connection(sync=True).hvals(name)

    def sync_sadd(self, name, values: list, time=None):
        con = self.get_connection(sync=True)
        con.sadd(name, *values)
        if time:
            con.expire(name, time)

    def sync_scard(self, name):
        return self.get_connection(sync=True).scard(name)

    def sync_sismember(self, name, value):
        return self.get_connection(sync=True).sismember(name, value)

    def sync_smembers(self, name):
        return self.get_connection(sync=True).smembers(name)

    def sync_spop(self, name, count):
        return self.get_connection(sync=True).spop(name, count)

    def sync_srem(self, name, *values):
        return self.get_connection(sync=True).srem(name, *values)



import inspect

swagger_json_project = []

swagger_json_class = []
python_column_type_dict = {
    "list": "array",
    "str": "string",
    "int": "integer",
    "dict": "object",
    "bool": "boolean",
    "float": "multipleOf"
}
definitions = {}


def parser_swagger(func, desc):
    if type(func).__name__ == "type":
        swagger_json_project.append(
            {
                "desc": desc,
                "route": func.__route_name__,
                "method_list": swagger_json_class.copy()
            }
        )
        swagger_json_class.clear()

    elif type(func).__name__ == "function":
        default_args = get_default_args(func)
        type_args = get_type_args(func)
        param_list = []
        func_params = func.__code__.co_varnames
        if func.__http_method__ == "get":
            for i in range(1, func.__code__.co_argcount):
                k = func_params[i]
                param = {
                    "required": True,
                    "name": k,
                    "in": "query",
                }
                if default_args.get(k):
                    param["required"] = False
                    param["default"] = default_args[k]["default"]
                param_list.append(param)
            swagger_json_class.append({
                "route": func.__route_name__,
                "summary": "Place an order for a pet",

                "operationId": "placeOrder",
                "consumes": ["application/json"],
                "produces": ["application/json", "application/xml"],
                "parameters": param_list,
                "description": desc,
                "http_method": func.__http_method__
            })
        elif func.__http_method__ == "post":

            schema = {
                "type": "object",
                "properties": {
                }
            }
            for i in range(1, func.__code__.co_argcount):
                k = func_params[i]
                param_type = python_column_type_dict[type_args.get(k, {}).get("type", str).__name__]
                if param_type in ["string", "integer"]:
                    schema["properties"][k] = {
                        "type": param_type,
                        "enum": [default_args.get(k, {}).get("default", "string")]
                    }
                elif param_type in ["array"]:
                    schema["properties"][k] = {
                        "type": "array",
                        "xml": {"wrapped": True},
                        "items": {"type": "string"},
                        "enum": [default_args.get(k, {}).get("default", "string")]
                    }
                elif param_type in ["object"]:
                    schema["properties"][k] = {
                        "type": "object",
                        "properties": {

                        }
                    }

            param = {
                "required": True,
                "name": "body",
                "in": "body",
                "schema": schema
            }
            swagger_json_class.append({
                "route": func.__route_name__,
                "parameters": [param],
                "description": desc,
                "http_method": func.__http_method__
            })
        else:
            pass


def get_type_args(func):
    signature = inspect.signature(func)
    return {
        k: {
            "type": v.annotation
        }
        for k, v in signature.parameters.items()
        if v.annotation is not inspect.Parameter.empty
    }


def get_default_args(func):
    signature = inspect.signature(func)
    return {
        k: {
            "default": v.default
        }
        for k, v in signature.parameters.items()
        if v.default is not inspect.Parameter.empty
    }



import base64

from Crypto.PublicKey import RSA as Rsa
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
from Crypto import Random


class RSA:
    @staticmethod
    def generate_key():
        _random_generator = Random.new().read
        # rsa算法生成实例
        _rsa = Rsa.generate(1024, _random_generator)
        # 私钥的生成
        _private_key = _rsa.exportKey()
        # 公钥的生成
        _public_key = _rsa.publickey().exportKey()
        key = {
            "private_key": _private_key,
            "public_key": _public_key
        }
        return key

    @staticmethod
    def encrypt(string: str, public_key: str, encoding='utf-8'):
        rsa_key = Rsa.importKey(public_key)
        cipher = Cipher_pkcs1_v1_5.new(rsa_key)
        encrypt_string = base64.b64encode(cipher.encrypt(string.encode(encoding))).decode(encoding)
        return encrypt_string

    @staticmethod
    def decrypt(string: str, private_key: str, encoding='utf-8'):
        rsa_key = Rsa.importKey(private_key)
        cipher = Cipher_pkcs1_v1_5.new(rsa_key)
        decrypt_string = cipher.decrypt(base64.b64decode(string.encode(encoding)), "解密失败").decode(encoding)
        return decrypt_string



def alchemy_default_to_dict(params, data, repetition=False):
    data_list = []
    key_list = []
    if repetition:
        for arg in params:
            if arg.key:
                key_list.append(arg.key)
            else:
                key_list.append(arg.name)
    else:
        for arg in params:
            arg = str(arg)
            if "(" in arg and ")" in arg:
                key_list.append(arg.split(".")[-1][:-1])
            else:
                key_list.append(arg.split(".")[-1])
    if isinstance(data, list):
        for d in data:
            dict_data = dict(zip(key_list, d))
            data_list.append(dict_data)
        return data_list
    else:
        if data:
            return dict(zip(key_list, data))
        else:
            return {}


def sqlalchemy_paging(Query, limit_number, offset_number):
    data_list = Query.limit(limit_number).offset(offset_number).all()
    data_count = Query.count()
    return {"count": data_count, "dataSource": data_list}



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/1 9:43 上午
# Copyright (C) 2021 The lesscode Team

class ConditionWrapper:
    """
    查询SQL语句条件包装类，用于组装查询SQL语句
    """

    def __init__(self, table, column="*"):
        self.table = table
        self.column = column
        self.conditions: list = []
        self.groups = ""
        self.order: list = []

    @property
    def or_(self):
        self.conditions.append(("OR", None, None))
        return self

    @property
    def and_(self):
        self.conditions.append(("AND", None, None))
        return self

    def eq(self, column, value):
        """
        添加条件 =
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("=", column, value))
        return self

    def eq_all(self, params: list):
        """
        添加多个条件 =
        :param params: 参数[("column","value")]
        :return:
        """
        if params:
            [self.eq(*item) for item in params if item]
        return self

    def ne(self, column, value):
        """
        添加条件 <>
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("<>", column, value))
        return self

    def gt(self, column, value):
        """
        添加条件 大于 >
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value:
            self.conditions.append((">", column, value))
        return self

    def ge(self, column, value):
        """
        添加条件 大于等于 >=
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append((">=", column, value))
        return self

    def lt(self, column, value):
        """
        添加条件 小于 <
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("<", column, value))
        return self

    def le(self, column, value):
        """
        添加条件 小于等于 <=
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("<=", column, value))
        return self

    def between(self, column, value1, value2):
        """
        添加条件 BETWEEN 值1 AND 值2
        :param column: 列名
        :param value1:
        :param value2:
        :return:
        """
        if value1 is None:
            self.ge(column, value1)
        elif value2 is None:
            self.lt(column, value2)
        else:
            self.conditions.append(("BETWEEN", column, [value1, value2]))
        return self

    def not_between(self, column, value1, value2):
        """
        添加条件 NOT BETWEEN 值1 AND 值2
        :param column: 列名
        :param value1:
        :param value2:
        :return:
        """
        if value1 is None:
            self.lt(column, value1)
        elif value2 is None:
            self.ge(column, value2)
        else:
            self.conditions.append(("NOT_BETWEEN", column, [value1, value2]))
        return self

    def like(self, column, value):
        """
        添加条件 LIKE '%值%'
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("LIKE", column, value))
        return self

    def notLike(self, column, value):
        """
        添加条件 NOT LIKE '%值%'
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("NOT_LIKE", column, value))
        return self

    def like_left(self, column, value):
        """
        添加条件 LIKE '%值'
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("LIKE_LEFT", column, value))
        return self

    def like_right(self, column, value):
        """
        添加条件 LIKE '值%'
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("LIKE_RIGHT", column, value))
        return self

    def is_null(self, column):
        """
        添加条件 字段 IS NULL
        :param column: 列名
        :return:
        """
        self.conditions.append(("IS_NULL", column, None))
        return self

    def is_not_null(self, column):
        """
        添加条件 字段 IS NOT NULL
        :param column: 列名
        :return:
        """
        self.conditions.append(("IS_NOT_NULL", column, None))
        return self

    def in_(self, column, value):
        """
        添加条件 字段 IN (v0, v1, ...)
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("IN", column, value))
        return self

    def not_in(self, column, value):
        """
        添加条件 NOT IN (v0, v1, ...)
        :param column: 列名
        :param value: 参考值
        :return:
        """
        if value is not None:
            self.conditions.append(("NOT_IN", column, value))
        return self

    def group(self, column):
        """
        添加条件 分组：GROUP BY 字段
        :param column: 列名
        :return:
        """
        self.groups = column
        return self

    def order_asc(self, column):
        """
        添加条件 排序：ORDER BY 字段, ... ASC
        :param column: 列名
        :return:
        """
        self.order.append((column, "ASC"))
        return self

    def order_desc(self, column):
        """
        添加条件 排序：ORDER BY 字段, ... DESC
        :param column: 列名
        :return:
        """
        self.order.append((column, "DESC"))
        return self



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/9 4:47 下午
# Copyright (C) 2021 The lesscode Team

from typing import Tuple


class StatusCode:
    """
    StatusCode 统一请求返回状态码
    # A表示错误来源于用户，比如参数错误，用户安装版本过低，用户支付超时等问题；
    # B表示错误来源于当前系统，往往是业务逻辑出错，或程序健壮性差等问题；
    # C表示错误来源于第三方服务
    """

    def __init__(self, message=None):
        self.message = message

    # *响应服务请求的状态码与说明

    # 通用状态码
    SUCCESS = ("00000", "请求成功")
    FAIL = ("99999", "请求失败")
    # A00开头的状态码
    USER_VALIDATE_FAIL = ("A0001", "用户端错误")
    # A01开头的状态码
    USER_REGISTER_FAIL = ("A0100", "用户注册错误")
    USER_NAME_VALIDATE_FAIL = ("A0110", "用户名校验失败")
    USER_NAME_EXIST = ("A0111", "用户名已存在")
    USER_NAME_INVALID = ("A0112", "用户名包含特殊字符")
    PHONE_ALREADY_REGISTER = ("A0113", "手机号已经注册")
    PASSWORD_VALIDATE_FAIL = ("A0120", "密码错误")
    PASSWORD_LENGTH_VALID = ("A0121", "密码长度不够")
    SHORT_MESSAGE_VALID_FAIL = ("A0130", "短信验证码错误")
    VALIDATE_CODE_ERROR = ("A0131", "验证码错误！")
    # A02开头的状态码
    USER_LOGIN_EXCEPTION = ("A0200", "用户登录异常")
    USER_ACCOUNT_NOT_EXIST = ("A0201", "用户不存在")
    USER_ACCOUNT_EXPIRE = ("A0202", "账号已失效，请联系管理员！")
    USER_ACCOUNT_PASSWORD_EXPIRE = ("A0203", "密码已失效，请修改密码，重新登录")
    USER_ACCOUNT_EXCEEDED_ERRORS = ("A0204", "密码错误次数过多，账号已锁定，请稍后重试")
    USER_ACCOUNT_DORMANCY = ("A0205", "账号处于休眠状态，请联系管理员！")
    USER_NOT_LOGIN_IN = ("A0206", "用户未登录")
    USER_ROLE_EXPIRE = ("A0207", "账号角色已失效，请联系管理员！")
    # A03开头的状态码
    REQUEST_PARAM_ERROR = ("A0300", "用户请求参数错误")
    INVALID_USER_INPUT = ("A0301", "无效的用户输入")
    INSECURE_TRANSPORT_ERROR = ("A0302", "OAuth 2 必须使用 https.")
    INVALID_REQUEST_ERROR = ("A0303", "无效的请求")
    INVALID_CLIENT_ERROR = ("A0304", "无效客户端信息")
    INVALID_GRANT_ERROR = ("A0305", "无效授权类型")
    UNAUTHORIZED_CLIENT_ERROR = ("A0306", "未授权客户端信息")
    UNSUPPORTED_RESPONSE_TYPE_ERROR = ("A0307", "不支持的响应类型")
    UNSUPPORTED_GRANT_TYPE_ERROR = ("A0308", "不支持的授权类型")
    INVALID_SCOPE_ERROR = ("A0309", "请求的作用域无效、未知或格式不正确")
    ACCESS_DENIED_ERROR = ("A0310", "请求被拒绝")
    MISSING_AUTHORIZATION_ERROR = ("A0311", "请求HEADERS中缺少 AUTHORIZATION 参数")
    UNSUPPORTED_TOKEN_TYPE_ERROR = ("A0312", "不支持的TOKEN类型")
    MISSING_CODE_EXCEPTION = ("A0313", '请求响应中缺少 "CODE"')
    MISSING_TOKEN_EXCEPTION = ("A0314", '请求响应中缺少 "ACCESS_TOKEN"')
    MISSING_TOKEN_TYPE_EXCEPTION = ("A0315", '请求响应中缺少 "TOKEN_TYPE"')
    MISMATCHING_STATE_EXCEPTION = ("A0316", "CSRF告警！请求和响应中的状态不相等")
    INVALID_TOKEN = ("A0317", "token已失效")
    FREQUENCY_LIMIT = ("A0318", "请求次数超限")
    WEIXIN_UNBOUND = ("A0319", "微信未绑定")
    WEIXIN_BOUNDED = ("A0320", "微信已绑定")
    INVALID_TIME_STAMP = ("A0321", "非法的时间戳参数")
    USER_INPUT_INVALID = ("A0322", "用户输入内容非法")
    UNSUPPORTED_SERVICE_TYPE_ERROR = ("A0323", "不支持的服务类型")
    FEISHU_UNBOUND = ("A0324", "飞书未绑定")
    FEISHU_BOUNDED = ("A0325", "飞书已绑定")
    WEIXIN_BOUNDED_OTHER = ("A0326", "微信已绑定在其他帐号了")

    @staticmethod
    def REQUIRED_PARAM_IS_EMPTY(message):
        """
        用于自定义消息提示信息
        :param message: 替换的消息内容
        :return:
        """
        return "A0399", "请求缺少必要参数:{}".format(message)

    # A04开头的状态码
    VALIDATE_CODE_EXPIRE = ("A0400", "验证码过期")
    FORM_VALIDATE_FAIL = ("A0401", "表单校验失败")
    PARAM_VALIDATE_FAIL = ("A0402", "参数校验失败")
    PARAM_BIND_FAIL = ("A0403", "参数绑定失败")
    PHONE_NUM_NOT_FOUND = ("A0404", "找不到该用户，手机号码有误")

    # A05开头的状态码

    # B00开头的状态码
    @staticmethod
    def BUSINESS_FAIL(message):
        """
        用于自定义消息提示信息
        :param message: 替换的消息内容
        :return:
        """
        return "B0000", "{}".format(message)

    ACCESS_DENIED = ("B0001", "访问权限不足")
    RESOURCE_DISABLED = ("B0002", "资源被禁用")
    RESOURCE_NO_AUTHORITY = ("B0003", "该资源未定义访问权限")

    # B01开头的状态码
    TIMEOUT = ("B0100", "系统执行超时")
    # B02开头的状态码
    STRIKE_RECOVERY = ("B0200", "系统容灾系统被触发")
    # B03开头的状态码

    # B04开头的状态码
    RESOURCE_NOT_FOUND = ("B0404", "访问资源不存在")

    # B05开头的状态码
    @staticmethod
    def INTERNAL_SERVER_ERROR(message):
        """
        用于自定义消息提示信息
        :param message: 替换的消息内容
        :return:
        """
        return "B0500", "服务器内部错误:{}".format(message)

    @staticmethod
    def RESOURCE_EXIST(message):
        """
        用于自定义消息提示信息
        :param message: 替换的消息内容
        :return:
        """
        return "B0501", "{}已存在,确保唯一性，不可重复".format(message)

    # C00开头的状态码
    RPC_INVOKE_ERROR = ("C0001", "调用第三方服务出错")
    SERVER_ERROR = ("C0002", "服务器内部错误")
    UNKNOWN_ERROR = ("C0003", "未知异常")

    # C01开头的状态码

    # C02开头的状态码

    # C03开头的状态码

    # C04开头的状态码

    # C05开头的状态码

    @staticmethod
    def COMMON_CODE_MESSAGE(status_code, message):
        """
        用于自定义消息提示信息
        :param message: 替换的消息内容
        :return:
        """
        return status_code, "{}".format(message)



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/14 3:44 下午

class DBFunction:
    def __init__(self, function_body, value):
        self.function_body = function_body
        self.value = value



class SHA256:
    @staticmethod
    def encrypt(string: str):
        import hashlib
        sha = hashlib.sha256(string.encode('utf-8'))
        encrypt_string = sha.hexdigest()
        return encrypt_string



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/4 11:00 上午
# Copyright (C) 2021 The lesscode Team

import threading

import tornado.options
from requests import post, get
from tornado.web import RequestHandler

from lesscode.db.redis.redis_helper import RedisHelper
from lesscode.utils.doc.interface_doc_handler import parser_swagger
from lesscode.web.business_exception import BusinessException
from lesscode.web.status_code import StatusCode
from tornado.options import define

define("route_prefix", default="", type=str, help="路由前缀")
tornado.options.options.route_prefix = ""


class RouterMapping(list):
    """
    RouterMapping 类用于存储访问路径"URL"与处理器之间对应关系集合
    直接继承list
    """

    _instance_lock = threading.Lock()

    def __init__(self):
        super(RouterMapping, self).__init__()
        # 存放类名、处理方法、url 的元祖集合 url为方法上指定路径
        self.dynamicMethods = []
        # 存放url与具体业务处理方法的映射关系（url，function）
        self.handlerMapping = []

    @classmethod
    def instance(cls):
        if not hasattr(RouterMapping, "_instance"):
            with RouterMapping._instance_lock:
                if not hasattr(RouterMapping, "_instance"):
                    RouterMapping._instance = RouterMapping()
        return RouterMapping._instance


def Handler(url: str, desc=""):
    """
    RequestHandler对应路径注册装饰器，完成处理类与url对应注册。
    :param prefix: 路由前缀
    :param url:
    :return:
    """
    route_prefix = tornado.options.options.route_prefix

    def wrapper(cls):
        # 验证是否为RequestHandler 子类，仅注册其子类
        if not issubclass(cls, RequestHandler):
            raise RuntimeError("Handler注释器只能装饰在RequestHandler子类上")
        # 通过类名查找对应该类下的所有注册方法信息
        res = [item for item in RouterMapping.instance().dynamicMethods
               if item[0] == cls.__name__]
        for item in res:
            # 处理类 RequestHandler 子类 使用时统一继承BaseHandler
            handler = item[1]
            # 全路径 Handler+Mapping 组合
            full_url = route_prefix + url + item[2]
            # 判断是否存在重复注册情况，重复情况直接抛出异常
            if [router for router in RouterMapping.instance() if full_url in router]:
                raise BusinessException(StatusCode.RESOURCE_EXIST(f'路由"{full_url}"'))
            # 存储URL与 RequestHandler 子类对应关系，用于提供给Tornado注册使用
            RouterMapping.instance().append((full_url, cls))
            # 存储URL与处理方法的对应关系，用于调用分发使用
            RouterMapping.instance().handlerMapping.append((full_url, handler))
            if tornado.options.options.rms_register_enable:
                client_name = item[3] if item[3] else tornado.options.options.application_name
                title = item[4] if item[4] else full_url
                en_name = item[5] if item[5] else full_url
                access = item[6] if item[6] else 0
                client_res = get(url=f"{tornado.options.options.rms_register_server}/upms/client/fetchall",
                                 params={"client_name": client_name}).json()
                if not client_res.get("data"):
                    post(url=f"{tornado.options.options.rms_register_server}/upms/client/insert",
                         json={"client_name": client_name})
                    client_res = get(url=f"{tornado.options.options.rms_register_server}/upms/client/fetchall",
                                     params={"client_name": client_name}).json()
                client_id = client_res.get("data")[0].get("id")
                resource_res = get(
                    url=f"{tornado.options.options.rms_register_server}/upms/resource/fetchall",
                    params={"client_name": client_name, "resource_type": 2, "url": full_url}).json()
                if not resource_res.get("data"):
                    res = post(url=f"{tornado.options.options.rms_register_server}/upms/resource/insert",
                               json={"client_id": client_id, "resource_type": 2, "url": full_url, "label": title,
                                     "en_name": en_name, "access": access}).json()
                    if res.get("status") == "00000":
                        print(f"add url={full_url} to rms success")
                    else:
                        print(f"add url={full_url} to rms fail")
        cls.__route_name__ = url
        try:
            parser_swagger(cls, desc)
        except:
            pass

        return cls

    return wrapper


def GetMapping(url: str = "", client_name: str = None, title: str = None, en_name: str = None, access: str = 1,
               interface_desc: str = ""):
    """
    用于类名、处理方法、url 的元祖集合注册处理，暂时GetMapping与PostMapping实现保持一致，预留入口为后期扩展提供支持。
    :param client_name:
    :param access:
    :param en_name:
    :param title:
    :param url:
    :return:
    """

    def wrapper(func):
        # 组合  类名、处理方法、url 的元祖 （url仅代表方法上Mapping装饰器的参数）
        path = url
        if not path:
            path = "/{}".format(func.__name__)
        RouterMapping.instance().dynamicMethods.append(
            (func.__qualname__.replace('.' + func.__name__, ''), func, path, client_name, title, en_name, access))
        func.__route_name__ = path
        func.__http_method__ = "get"
        try:
            parser_swagger(func, (title if title else "") + interface_desc)
        except:
            pass
        return func

    return wrapper


def PostMapping(url: str = "", client_name: str = None, title: str = None, en_name: str = None, access: str = 1,
                interface_desc: str = ""):
    """
    用于类名、处理方法、url 的元祖集合注册处理，暂时GetMapping与PostMapping实现保持一致，预留入口为后期扩展提供支持。
    :param client_name:
    :param access:
    :param en_name:
    :param title:
    :param url:
    :return:
    """

    def wrapper(func):
        # 组合  类名、处理方法、url 的元祖 （url仅代表方法上Mapping装饰器的参数）
        path = url
        if not path:
            path = "/{}".format(func.__name__)
        RouterMapping.instance().dynamicMethods.append(
            (func.__qualname__.replace('.' + func.__name__, ''), func, path, client_name, title, en_name, access))
        func.__route_name__ = path
        func.__http_method__ = "post"
        try:
            parser_swagger(func, (title if title else "") + interface_desc)
        except:
            pass

        return func

    return wrapper



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/3/2 9:14 上午
# Copyright (C) 2021 The lesscode Team
import json
import random

import requests
from requests.auth import HTTPBasicAuth


class EsRequest:

    def __init__(self, host, port, user, password):
        # 主机地址
        self.host = host
        # 端口号
        self.port = port
        # 用户名
        self.user = user
        # 密码
        self.password = password
        self.auth = HTTPBasicAuth(user, password)
        host_str = host.split(",")
        self.hosts = [host for host in host_str]

    def es_selector_way(self, url_func_str, param_dict, find_condition):
        res = None
        # 随机打乱列表
        random.shuffle(self.hosts)
        for host in self.hosts:
            param_dict["host"] = host
            param_dict["port"] = self.port
            url = url_func_str(**param_dict)
            try:
                res = self.format_es_post(url, find_condition)
                break
            except:
                continue
        return res

    def format_es_post(self, url, body):
        """
        发送http请求
        :param url:
        :param body:
        :return:
        """
        r = requests.post(
            url,
            data=json.dumps(body),
            headers={'content-type': "application/json"},
            auth=self.auth
        )
        res = r.json()
        return res

    def format_scroll_url(self, host=None, port=None, route_key=None, scroll=None):
        return "http://{}:{}/{}/_search?scroll={}".format(host if host else self.host,
                                                          port if port else self.port, route_key, scroll)

    def format_scroll_id_url(self, host=None, port=None, ):
        return "http://{}:{}/_search/scroll".format(host if host else self.host,
                                                    port if port else self.port)

    def format_es_post_url(self, host=None, port=None, route_key=None):
        return "http://{}:{}/{}/_search".format(host if host else self.host,
                                                port if port else self.port, route_key)

    def close(self):
        pass




# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/26 3:56 下午
# Copyright (C) 2021 The lesscode Team
import json

import neo4j.data
from tornado.options import options

import logging
import traceback

from neo4j import WorkspaceConfig

from lesscode.db.page import Page
from lesscode.utils.encryption_algorithm import AES


class Neo4jHelper:
    """
    Neo4j   数据库操作实现
    """

    def __init__(self, pool, access=neo4j.READ_ACCESS):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):

            self.pool, self.dialect = options.database[pool]
            self.pool = self.pool.session(database=pool, default_access_mode=access)
        else:
            self.pool = pool

    def __repr__(self):
        printer = 'o(>﹏<)o ......Neo4j old driver "{0}" carry me fly...... o(^o^)o'.format(self.pool)
        return printer

    async def listreader(self, cypher, keys):
        """
        Read data from Neo4j in specified cypher.
        Read and parse data straightly from cypher field result.
        :param cypher: string
            Valid query cypher statement.
        :param keys: list
            Cypher query columns to return.
        :return: list
            Each returned record constructs a list and stored in a big list, [[...], [...], ...].
        """
        tx = await self.pool.begin_transaction()
        data = []
        result = await tx.run(cypher)
        async for record in result:
            rows = []
            for key in keys:
                rows.append(record[key])
            data.append(rows)
        await self.pool.close()
        return data

    async def dictreader(self, cypher):
        """
        Read data from Neo4j in specified cypher.
        The function depends on constructing dict method of dict(key = value) and any error may occur if the "key" is invalid to Python.
        you can choose function dictreaderopted() below to read data by hand(via the args "keys").
        :param cypher: string
            Valid query cypher statement.
        :return: list
            Each returned record constructs a dict in "key : value" pairs and stored in a big list, [{...}, {...}, ...].
        """
        tx = await self.pool.begin_transaction()
        data = []
        result = await tx.run(cypher)
        async for record in result.records():
            item = {}
            for args in str(record).split('>')[0].split()[1:]:
                exec
                "item.update(dict({0}))".format(args)
            data.append(item)
        await self.pool.close()
        return data

    async def dictreaderopted(self, cypher, keys=None):
        """
        Optimized function of dictreader().
        Read and parse data straightly from cypher field result.
        :param cypher: string
            Valid query cypher statement.
        :param keys: list, default : none(call dictreader())
            Cypher query columns to return.
        :return: list.
            Each returned record constructs an dict in "key : value" pairs and stored in a list, [{...}, {...}, ...].
        """
        if not keys:
            return await self.dictreader(cypher)
        else:
            tx = await self.pool.begin_transaction()
            data = []
            result = await tx.run(cypher)
            async for record in result:
                item = {}
                for key in keys:
                    item.update({key: record[key]})
                data.append(item)
            await self.pool.close()
            return data

    async def cypherexecuter(self, cypher):
        """
        Execute manipulation into Neo4j in specified cypher.
        :param cypher: string
            Valid handle cypher statement.
        :return: none.
        """
        async with self.pool.begin_transaction() as tx:
            res = await tx.run(cypher)
        await self.pool.close()
        return res

    async def parse_relation_data(self, cql, result_list, id_key="id", name_key="name"):
        data = []
        num = 0
        stock_strike = True
        while stock_strike and num < 3:
            try:
                res = await self.listreader(cql, result_list)
                for item in res:
                    level = 1
                    if len(item) > 1:
                        if isinstance(item[1], list):
                            for item_item in item[1]:
                                relation_item = parse_relation_item(item_item, id_key, name_key, level)
                                level = level + 1
                                data.append(relation_item)
                        else:
                            relation_item = parse_relation_item(item[1], id_key, name_key, level)
                            data.append(relation_item)
                    else:
                        if isinstance(item[0], list):
                            for item_item in item[0]:
                                relation_item_list = parse_relation_item_more_relation(item_item, id_key, name_key)
                                data = data + relation_item_list
                        else:
                            relation_item_list = parse_relation_item_more_relation(item[0], id_key, name_key)
                            data = data + relation_item_list
                stock_strike = False
            except Exception:
                logging.error(traceback.format_exc())
            num = num + 1
        return data

    async def create_node(self, node_name: str, label_name: str, node_property: dict = {}):
        sql = f"CREATE ({node_name}:{label_name} {dict_str(node_property)})"
        result = await self.execute_write(sql)
        return result

    async def create_node_relationship(self, relationship_list: list):
        sql = f"CREATE "
        sql += ",".join(relationship_list)
        result = await self.execute_write(sql)
        return result

    async def delete_node(self, node_name: str, label_name: str, match_property_dict: dict = {},
                          node_property_dict: dict = {}):
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        sql += f" DELETE {node_name}"
        result = await self.execute_write(sql)
        return result

    async def delete_property(self, node_name: str, label_name: str, delete_node_property_list: list,
                              match_property_dict: dict = {}, node_property_dict: dict = {}):
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        delete_node_property_str = ','.join([f'{node_name}.{p}' for p in delete_node_property_list])
        sql += f' REMOVE {delete_node_property_str} RETURN {node_name}'
        result = await self.execute_write(sql)
        return result

    async def update_node(self, node_name: str, label_name: str, new_node_property_dict: dict,
                          match_property_dict: dict = {}, node_property_dict: dict = {}):
        new_node_property_str = ",".join(
            ['%s.%s=%r' % (node_name, property_name, property_value) for property_name, property_value in
             new_node_property_dict.items()])
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        sql += f" SET {new_node_property_str} RETURN {node_name}"
        result = await self.execute_write(sql)
        return result

    async def delete_two_node_relationship(self, relationship_name: str, relationship: str, node1_name: str = None,
                                           label1_name: str = None, node1_match_property_dict: dict = {},
                                           node2_name: str = None, label2_name: str = None,
                                           node2_match_property_dict: dict = {}):
        if node1_name:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({node1_name}:{label1_name}'
            elif node1_match_property_dict and not label1_name:
                sql = f'MATCH ({node1_name} {dict_str(node1_match_property_dict)}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({node1_name}:{label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
            sql += ")"
        else:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({label1_name}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
            sql += ")"
        node2_str = f'('
        if node2_name:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{node2_name}:{label2_name}'
            elif node2_match_property_dict and not label2_name:
                node2_str += f'{node2_name} {dict_str(node2_match_property_dict)}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{node2_name}:{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{label2_name}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
        node2_str += ')'
        sql += f'{relationship}{node2_str} DELETE {relationship_name}'
        result = await self.execute_write(sql)
        return result

    async def delete_node_and_all_relationship(self, node_name: str = None, label_name: str = None,
                                               match_property_dict: dict = {},
                                               node_property_dict: dict = {}):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if node_name:
            sql += f" DETACH DELETE {node_name}"
        else:
            sql += f" DETACH DELETE {label_name}"
        result = await self.execute_write(sql)
        return result

    async def query_node_relationship(self, relationship: str, node1_name: str = None, label1_name: str = None,
                                      node1_match_property_dict: dict = {},
                                      property_dict: dict = {}, node2_name: str = None, label2_name: str = None,
                                      node2_match_property_dict: dict = {}, result_list: list = None):
        if node1_name:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({node1_name}:{label1_name})'
            elif node1_match_property_dict and not label1_name:
                sql = f'MATCH ({node1_name} {dict_str(node1_match_property_dict)})'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({node1_name}:{label1_name} {dict_str(node1_match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({label1_name})'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({label1_name} {dict_str(node1_match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node2_str = f'('
        if node2_name:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{node2_name}:{label2_name}'
            elif node2_match_property_dict and not label2_name:
                node2_str += f'{node2_name} {dict_str(node2_match_property_dict)}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{node2_name}:{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{label2_name}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
        node2_str += ')'
        sql += relationship
        sql += node2_str
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if result_list:
            sql += f' RETURN {",".join(result_list)}'
        else:
            if node1_name and not node2_name and label2_name:
                sql += f' RETURN {node1_name},{label2_name}'
            elif node1_name and node2_name:
                sql += f' RETURN {node1_name},{node2_name}'
            elif not node1_name and label1_name and node2_name:
                sql += f' RETURN {label1_name},{node2_name}'
            elif not node1_name and label1_name and not node2_name and label2_name:
                sql += f' RETURN {label1_name},{label2_name}'
            else:
                raise Exception("node1 and node2 are not exist")
        results = await self.pool.read_transaction(self.query_work, sql)
        await self.pool.close()
        records = []
        for record in results:
            records.append(parse_record(record, flag=True))
        return records

    async def query_all_node(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                             node_property_dict: dict = {}, result_list: list = None):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if result_list:
            sql += f' RETURN {",".join(result_list)}'
        else:
            if node_name:
                sql += f' RETURN {node_name}'
            else:
                sql += f' RETURN {label_name}'
        results = await self.pool.read_transaction(self.query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=False))
        await self.pool.close()
        return records

    async def query_node_count(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                               node_property_dict: dict = {}):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        sql = sql + ' RETURN COUNT(*)'
        result = await self.pool.read_transaction(self.query_work, sql)
        count = result[0].value()
        return count

    async def query_node_page(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                              node_property_dict: dict = {}, result_list: list = None,
                              page_num: int = 1, page_size: int = 10):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        page_num = page_num if page_num > 1 else 1
        num = (page_num - 1) * page_size
        total = await self.query_node_count(node_name, label_name, match_property_dict, node_property_dict)
        if result_list:
            sql += f' RETURN {",".join(result_list)} SKIP {num} LIMIT {page_size}'
        else:
            if node_name:
                sql += f' RETURN {node_name} SKIP {num} LIMIT {page_size}'
            else:
                sql += f' RETURN {label_name} SKIP {num} LIMIT {page_size}'
        results = await self.pool.read_transaction(self.query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=False))
        await self.pool.close()
        return Page(records=records, current=page_num, page_size=page_size, total=total).__dict__

    async def execute_fetchall(self, sql, flag=True):
        results = await self.pool.read_transaction(self.query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=flag))
        return records

    async def query_work(self, tx, sql):
        result = await tx.run(sql)
        return [record async for record in result]

    async def write_work(self, tx, sql):
        result = await tx.run(sql)
        return await result.consume()

    async def execute_write(self, sql):
        summary = await self.pool.write_transaction(self.sync_write_work, sql)
        res = eval(summary.counters.__repr__())
        await self.pool.close()
        return res

    def sync_listreader(self, cypher, keys):
        """
        Read data from Neo4j in specified cypher.
        Read and parse data straightly from cypher field result.
        :param cypher: string
            Valid query cypher statement.
        :param keys: list
            Cypher query columns to return.
        :return: list
            Each returned record constructs a list and stored in a big list, [[...], [...], ...].
        """
        data = []
        with self.pool.begin_transaction() as tx:

            result = tx.run(cypher)
            for record in result:
                rows = []
                for key in keys:
                    rows.append(record[key])
                data.append(rows)
        self.pool.close()
        return data

    def sync_dictreader(self, cypher):
        """
        Read data from Neo4j in specified cypher.
        The function depends on constructing dict method of dict(key = value) and any error may occur if the "key" is invalid to Python.
        you can choose function dictreaderopted() below to read data by hand(via the args "keys").
        :param cypher: string
            Valid query cypher statement.
        :return: list
            Each returned record constructs a dict in "key : value" pairs and stored in a big list, [{...}, {...}, ...].
        """
        data = []
        with self.pool.begin_transaction() as tx:
            for record in tx.run(cypher).records():
                item = {}
                for args in str(record).split('>')[0].split()[1:]:
                    exec
                    "item.update(dict({0}))".format(args)
                data.append(item)
        return data

    def sync_dictreaderopted(self, cypher, keys=None):
        """
        Optimized function of dictreader().
        Read and parse data straightly from cypher field result.
        :param cypher: string
            Valid query cypher statement.
        :param keys: list, default : none(call dictreader())
            Cypher query columns to return.
        :return: list.
            Each returned record constructs an dict in "key : value" pairs and stored in a list, [{...}, {...}, ...].
        """
        if not keys:
            return self.sync_dictreader(cypher)
        else:
            data = []
            with self.pool.begin_transaction() as tx:
                result = tx.run(cypher)
                for record in result:
                    item = {}
                    for key in keys:
                        item.update({key: record[key]})
                    data.append(item)
            return data

    def sync_cypherexecuter(self, cypher):
        """
        Execute manipulation into Neo4j in specified cypher.
        :param cypher: string
            Valid handle cypher statement.
        :return: none.
        """
        with self.pool.begin_transaction() as tx:
            return tx.run(cypher)

    def sync_parse_relation_data(self, cql, result_list, id_key="id", name_key="name"):
        data = []
        num = 0
        stock_strike = True
        while stock_strike and num < 3:
            try:
                res = self.sync_listreader(cql, result_list)
                for item in res:
                    level = 1
                    if len(item) > 1:
                        if isinstance(item[1], list):
                            for item_item in item[1]:
                                relation_item = parse_relation_item(item_item, id_key, name_key, level)
                                level = level + 1
                                data.append(relation_item)
                        else:
                            relation_item = parse_relation_item(item[1], id_key, name_key, level)
                            data.append(relation_item)
                    else:
                        if isinstance(item[0], list):
                            for item_item in item[0]:
                                relation_item_list = parse_relation_item_more_relation(item_item, id_key, name_key)
                                data = data + relation_item_list
                        else:
                            relation_item_list = parse_relation_item_more_relation(item[0], id_key, name_key)
                            data = data + relation_item_list
                stock_strike = False
            except Exception:
                logging.error(traceback.format_exc())
            num = num + 1
        return data

    def sync_create_node(self, node_name: str, label_name: str, node_property: dict = {}):
        sql = f"CREATE ({node_name}:{label_name} {dict_str(node_property)})"
        result = self.sync_execute_write(sql)
        return result

    def sync_create_node_relationship(self, relationship_list: list):
        sql = f"CREATE "
        sql += ",".join(relationship_list)
        result = self.sync_execute_write(sql)
        return result

    def sync_delete_node(self, node_name: str, label_name: str, match_property_dict: dict = {},
                         node_property_dict: dict = {}):
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        sql += f" DELETE {node_name}"
        result = self.sync_execute_write(sql)
        return result

    def sync_delete_property(self, node_name: str, label_name: str, delete_node_property_list: list,
                             match_property_dict: dict = {}, node_property_dict: dict = {}):
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        delete_node_property_str = ','.join([f'{node_name}.{p}' for p in delete_node_property_list])
        sql += f' REMOVE {delete_node_property_str} RETURN {node_name}'
        result = self.sync_execute_write(sql)
        return result

    def sync_update_node(self, node_name: str, label_name: str, new_node_property_dict: dict,
                         match_property_dict: dict = {}, node_property_dict: dict = {}):
        new_node_property_str = ",".join(
            ['%s.%s=%r' % (node_name, property_name, property_value) for property_name, property_value in
             new_node_property_dict.items()])
        sql = f"MATCH ({node_name}:{label_name}"
        if match_property_dict:
            sql += f"{dict_str(match_property_dict)})"
        else:
            sql += ")"
        if node_property_dict:
            node_property_str = ",".join(
                ['%s=%r' % (property_name, property_value) for property_name, property_value in
                 node_property_dict.items()])
            sql += f' WHERE {node_property_str}'
        sql += f" SET {new_node_property_str} RETURN {node_name}"
        result = self.sync_execute_write(sql)
        return result

    def sync_delete_two_node_relationship(self, relationship_name: str, relationship: str, node1_name: str = None,
                                          label1_name: str = None, node1_match_property_dict: dict = {},
                                          node2_name: str = None, label2_name: str = None,
                                          node2_match_property_dict: dict = {}):
        if node1_name:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({node1_name}:{label1_name}'
            elif node1_match_property_dict and not label1_name:
                sql = f'MATCH ({node1_name} {dict_str(node1_match_property_dict)}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({node1_name}:{label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
            sql += ")"
        else:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({label1_name}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
            sql += ")"
        node2_str = f'('
        if node2_name:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{node2_name}:{label2_name}'
            elif node2_match_property_dict and not label2_name:
                node2_str += f'{node2_name} {dict_str(node2_match_property_dict)}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{node2_name}:{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{label2_name}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
        node2_str += ')'
        sql += f'{relationship}{node2_str} DELETE {relationship_name}'
        result = self.sync_execute_write(sql)
        return result

    def sync_delete_node_and_all_relationship(self, node_name: str = None, label_name: str = None,
                                              match_property_dict: dict = {},
                                              node_property_dict: dict = {}):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if node_name:
            sql += f" DETACH DELETE {node_name}"
        else:
            sql += f" DETACH DELETE {label_name}"
        result = self.sync_execute_write(sql)
        return result

    def sync_query_node_relationship(self, relationship: str, node1_name: str = None, label1_name: str = None,
                                     node1_match_property_dict: dict = {},
                                     property_dict: dict = {}, node2_name: str = None, label2_name: str = None,
                                     node2_match_property_dict: dict = {},
                                     result_list: list = None):
        if node1_name:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({node1_name}:{label1_name}'
            elif node1_match_property_dict and not label1_name:
                sql = f'MATCH ({node1_name} {dict_str(node1_match_property_dict)}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({node1_name}:{label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
            sql += ")"
        else:
            if label1_name and not node1_match_property_dict:
                sql = f'MATCH ({label1_name}'
            elif node1_match_property_dict and label1_name:
                sql = f'MATCH ({label1_name} {dict_str(node1_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
            sql += ")"
        node2_str = f'('
        if node2_name:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{node2_name}:{label2_name}'
            elif node2_match_property_dict and not label2_name:
                node2_str += f'{node2_name} {dict_str(node2_match_property_dict)}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{node2_name}:{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label2_name and not node2_match_property_dict:
                node2_str += f'{label2_name}'
            elif node2_match_property_dict and label2_name:
                node2_str += f'{label2_name} {dict_str(node2_match_property_dict)}'
            else:
                raise Exception("At least one of label_name and node_name")
        node2_str += ')'
        sql += relationship
        sql += node2_str
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if result_list:
            sql += f' RETURN {",".join(result_list)}'
        else:
            if node1_name and not node2_name and label2_name:
                sql += f' RETURN {node1_name},{label2_name}'
            elif node1_name and node2_name:
                sql += f' RETURN {node1_name},{node2_name}'
            elif not node1_name and label1_name and node2_name:
                sql += f' RETURN {label1_name},{node2_name}'
            elif not node1_name and label1_name and not node2_name and label2_name:
                sql += f' RETURN {label1_name},{label2_name}'
            else:
                raise Exception("node1 and node2 are not exist")
        results = self.pool.read_transaction(self.sync_query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=True))
        return records

    def sync_query_all_node(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                            node_property_dict: dict = {}, result_list: list = None):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        if result_list:
            sql += f' RETURN {",".join(result_list)}'
        else:
            if node_name:
                sql += f' RETURN {node_name}'
            else:
                sql += f' RETURN {label_name}'
        results = self.pool.read_transaction(self.sync_query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=False))
        return records

    def sync_query_node_count(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                              node_property_dict: dict = {}):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        sql = sql + ' RETURN COUNT(*)'
        count = self.pool.read_transaction(self.sync_query_work, sql)[0].value()
        return count

    def sync_query_node_page(self, node_name: str = None, label_name: str = None, match_property_dict: dict = {},
                             node_property_dict: dict = {}, result_list: list = None,
                             page_num: int = 1, page_size: int = 10):
        if node_name:
            if label_name and not match_property_dict:
                sql = f'MATCH ({node_name}:{label_name})'
            elif match_property_dict and not label_name:
                sql = f'MATCH ({node_name} {dict_str(match_property_dict)})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({node_name}:{label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and match_property_dict")
        else:
            if label_name and not match_property_dict:
                sql = f'MATCH ({label_name})'
            elif match_property_dict and label_name:
                sql = f'MATCH ({label_name} {dict_str(match_property_dict)})'
            else:
                raise Exception("At least one of label_name and node_name")
        node_property_str = ",".join(
            ['%s=%r' % (property_name, property_value) for property_name, property_value in
             node_property_dict.items()])
        if node_property_str:
            sql += f' WHERE {node_property_str}'
        page_num = page_num if page_num > 1 else 1
        num = (page_num - 1) * page_size
        total = self.sync_query_node_count(node_name, label_name, match_property_dict, node_property_dict)
        if result_list:
            sql += f' RETURN {",".join(result_list)} SKIP {num} LIMIT {page_size}'
        else:
            if node_name:
                sql += f' RETURN {node_name} SKIP {num} LIMIT {page_size}'
            else:
                sql += f' RETURN {label_name} SKIP {num} LIMIT {page_size}'
        results = self.pool.read_transaction(self.sync_query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=False))
        self.pool.close()
        return Page(records=records, current=page_num, page_size=page_size, total=total).__dict__

    def sync_query_work(self, tx, sql):
        return list(tx.run(sql))

    def sync_execute_fetchall(self, sql, flag=True):
        results = self.pool.read_transaction(self.sync_query_work, sql)
        records = []
        for record in results:
            records.append(parse_record(record, flag=flag))
        self.pool.close()
        return records

    def sync_write_work(self, tx, sql):
        return tx.run(sql).consume()

    def sync_execute_write(self, sql):
        summary = self.pool.write_transaction(self.sync_write_work, sql)
        res = eval(summary.counters.__repr__())
        self.pool.close()
        return res


def parse_relation_item(item, id_key, name_key, level=0):
    relation_item = {"start_node": parse_node_dict(item.start_node, id_key, name_key),
                     "end_node": parse_node_dict(item.end_node, id_key, name_key),
                     "type": item.type, "level": level, "properties": item._properties}
    return relation_item


def parse_relation_item_more_relation(item, id_key, name_key):
    relation_item_list = []
    for data in item._relationships:
        relation_item_list.append(parse_relation_item(data, id_key, name_key))
    return relation_item_list


def parse_node_dict(node, id_key, name_key):
    return {
        "id": AES.encrypt(options.aes_key, node._properties[id_key]),
        "name": node._properties[name_key],
        "label": list(node._labels)[0]
    }


def dict_str(data: dict):
    ls = []
    for k, v in data.items():
        ls.append('%s:%r' % (k, v))
    string = '{' + ','.join(ls) + '}'
    return string


def node2dict(node: neo4j.data.Node):
    return {"id": node.id, "labels": list(node.labels), "properties": node._properties}


def relationship2dict(relationship: neo4j.data.Relationship):
    return {"start_node": node2dict(relationship.start_node), "end_node": node2dict(relationship.end_node),
            "id": relationship.id,
            "type": relationship.type, "properties": relationship._properties}


def path2dict(path: neo4j.graph.Path):
    return {"start_node": node2dict(path.start_node), "end_node": node2dict(path.end_node),
            "nodes": [node2dict(n) for n in path.nodes],
            "relationships": [relationship2dict(r) for r in path.relationships]}


def parse_record(record: neo4j.data.Record, flag=True):
    _record = {"node": [], "relationship": [], "path": [], "other": []}
    if isinstance(record, neo4j.data.Record):
        for rd in record:
            if isinstance(rd, neo4j.data.Node):
                _record["node"].append(node2dict(rd))
            elif issubclass(rd.__class__, neo4j.data.Node):
                _record["node"].append(node2dict(rd))
            elif isinstance(rd, neo4j.data.Relationship):
                _record["relationship"].append(relationship2dict(rd))
            elif issubclass(rd.__class__, neo4j.data.Relationship):
                _record["relationship"].append(relationship2dict(rd))
            elif isinstance(rd, list) or isinstance(rd, tuple):
                for x in rd:
                    if isinstance(x, neo4j.data.Node):
                        _record["node"].append(node2dict(x))
                    elif issubclass(x.__class__, neo4j.data.Node):
                        _record["node"].append(node2dict(x))
                    elif isinstance(x, neo4j.data.Relationship):
                        _record["relationship"].append(relationship2dict(x))
                    elif issubclass(x.__class__, neo4j.data.Relationship):
                        _record["relationship"].append(relationship2dict(x))
                    elif isinstance(rd, neo4j.graph.Path):
                        _record["path"].append(path2dict(rd))
                    else:
                        _record["other"].append(rd)
            elif isinstance(rd, neo4j.graph.Path):
                _record["path"].append(path2dict(rd))
            else:
                _record["other"].append(rd)

    if not flag:
        _record.pop("relationship")
    return _record



import copy
import datetime
import functools
import json
import logging
import traceback

from lesscode.db.redis.redis_helper import RedisHelper
from tornado.options import options

# 装饰器
from util.JsonUtil import JSONEncoder


def Cache(ex=3600 * 8, cache_key=None):
    def cache_func(func):
        # 默认key生成方法：str(item)
        @functools.wraps(func)
        def cache_wrapper(*args, **params):
            return deal_cache(func, ex, cache_key, *args, **params)

        return cache_wrapper

    return cache_func


def deal_cache(func, ex, cache_key, *args, **params):
    # 获取缓存查询key
    params = dict(sorted(params.items(), key=lambda x: x[0]))
    func_name = str(func).split(" ")[1]
    if not cache_key:
        cache_key = format_insert_key(func_name, params)
    value = query_cache(cache_key)
    if value:
        data = value
    else:
        start = datetime.datetime.now()
        logging.info("[组件：{}]数据开始计算！".format(func_name))
        copy_params = copy.deepcopy(params)
        data = func(*args, **copy_params)

        # 插入缓存表
        insert_cache(data, ex, cache_key)
        logging.info("[组件：{}]数据缓存已刷新！用时{}".format(func_name, datetime.datetime.now() - start))

    return data


def query_cache(cache_key):
    if options.cache_module:
        data = RedisHelper("redis_con_1").sync_get(cache_key)
        if data:
            value = json.loads(data)
            return value
        else:
            logging.info("str_select_key为".format(cache_key))
            return False
    return False


def format_insert_key(func_name, params):
    str_insert_key = ""
    for k in params:
        if isinstance(params[k], dict):
            a = json.dumps(params[k])
        else:
            a = str("" if params[k] is None else params[k])

        str_insert_key = str_insert_key + a + "&"
    str_insert_key = str_insert_key[:-1]
    str_insert_key = func_name + "_" + str_insert_key

    return str_insert_key


def insert_cache(data, ex, cache_key):
    if options.cache_module:
        try:
            RedisHelper("redis_con_1").sync_set(cache_key, JSONEncoder().encode(data), ex=ex)
        except:
            logging.error(traceback.format_exc())



from base64 import b64encode, b64decode


class BASE64:
    @staticmethod
    def encrypt(string: str, encoding='utf-8'):
        encrypt_string = b64encode(string.encode(encoding)).decode(encoding)
        return encrypt_string

    @staticmethod
    def decrypt(string: str, encoding='utf-8'):
        decrypt_string = b64decode(string.encode(encoding)).decode(encoding)
        return decrypt_string
